@@ -1,8 +1,8 @@
 import sys
-from django.utils import six
 
 from dbsettings.values import Value
 from dbsettings.loading import register_setting, unregister_setting
+from dbsettings.management import mk_permissions
 
 __all__ = ['Group']
 
@@ -34,8 +34,7 @@ class GroupDescriptor(object):
         return self.group
 
 
-@six.add_metaclass(GroupBase)
-class Group(object):
+class Group(object, metaclass=GroupBase):
 
     def __new__(cls, verbose_name=None, copy=True, app_label=None):
         # If not otherwise provided, set the module to where it was executed
@@ -89,6 +88,9 @@ class Group(object):
             except AttributeError:
                 # Permissions were supplied as a tuple, so preserve that
                 cls._meta.permissions = tuple(cls._meta.permissions + (permission,))
+        # Django migrations runner cache properties
+        if hasattr(cls._meta, 'original_attrs'):
+            cls._meta.original_attrs['permissions'] = cls._meta.permissions
 
         # Finally, place the attribute on the class
         setattr(cls, name, GroupDescriptor(self, name))
