@@ -1,6 +1,7 @@
 from collections import OrderedDict
 from django.core.cache import cache
 
+from .signals import setting_changed
 
 __all__ = ['get_all_settings', 'get_setting', 'get_setting_storage',
            'register_setting', 'unregister_setting', 'set_setting_value']
@@ -36,8 +37,11 @@ def setting_in_db(module_name, class_name, attribute_name):
 
 def get_setting_storage(module_name, class_name, attribute_name):
     from dbsettings.models import Setting
-    key = _get_cache_key(module_name, class_name, attribute_name)
-    storage = cache.get(key)
+    from dbsettings.settings import USE_CACHE
+    storage = None
+    if USE_CACHE:
+        key = _get_cache_key(module_name, class_name, attribute_name)
+        storage = cache.get(key)
     if storage is None:
         try:
             storage = Setting.objects.get(
@@ -53,7 +57,8 @@ def get_setting_storage(module_name, class_name, attribute_name):
                 attribute_name=attribute_name,
                 value=setting_object.default,
             )
-        cache.set(key, storage)
+        if USE_CACHE:
+            cache.set(key, storage)
     return storage
 
 
@@ -68,9 +73,12 @@ def unregister_setting(setting):
 
 
 def set_setting_value(module_name, class_name, attribute_name, value):
+    from dbsettings.settings import USE_CACHE
     setting = get_setting(module_name, class_name, attribute_name)
     storage = get_setting_storage(module_name, class_name, attribute_name)
-    storage.value = setting.get_db_prep_save(value)
+    storage.value = setting.get_db_prep_save(value, oldvalue=storage.value)
     storage.save()
-    key = _get_cache_key(module_name, class_name, attribute_name)
-    cache.delete(key)
+    setting_changed.send(sender=setting, value=setting.to_python(value))
+    if USE_CACHE:
+        key = _get_cache_key(module_name, class_name, attribute_name)
+        cache.delete(key)
