@@ -1,5 +1,4 @@
 from __future__ import unicode_literals
-from django.utils import six
 
 import datetime
 from decimal import Decimal
@@ -29,11 +28,12 @@ class Value(object):
     creation_counter = 0
     unitialized_value = None
 
-    def __init__(self, description=None, help_text=None, choices=None, required=True, default=None):
+    def __init__(self, description=None, help_text=None, choices=None, required=True, default=None, widget=None):
         self.description = description
         self.help_text = help_text
         self.choices = choices or []
         self.required = required
+        self.widget = widget
         if default is None:
             self.default = self.unitialized_value
         else:
@@ -84,17 +84,20 @@ class Value(object):
 
     # Subclasses should override the following methods where applicable
 
+    def meaningless(self, value):
+        return value is None or value == ""
+
     def to_python(self, value):
         "Returns a native Python object suitable for immediate use"
         return value
 
-    def get_db_prep_save(self, value):
+    def get_db_prep_save(self, value, oldvalue=None):
         "Returns a value suitable for storage into a CharField"
-        return six.text_type(value)
+        return str(value)
 
     def to_editor(self, value):
         "Returns a value suitable for display in a form widget"
-        return six.text_type(value)
+        return str(value)
 
 ###############
 # VALUE TYPES #
@@ -122,7 +125,7 @@ class DecimalValue(Value):
     field = forms.DecimalField
 
     def to_python(self, value):
-        return Decimal(value)
+        return Decimal(value) if not self.meaningless(value) else None
 
 
 # DurationValue has a lot of duplication and ugliness because of issue #2443
@@ -149,8 +152,8 @@ class DurationValue(Value):
         except OverflowError:
             raise forms.ValidationError('The maximum allowed value is %s' % datetime.timedelta.max)
 
-    def get_db_prep_save(self, value):
-        return six.text_type(value.days * 24 * 3600 + value.seconds
+    def get_db_prep_save(self, value, **kwargs):
+        return str(value.days * 24 * 3600 + value.seconds
                              + float(value.microseconds) / 1000000)
 
 
@@ -158,32 +161,31 @@ class FloatValue(Value):
     field = forms.FloatField
 
     def to_python(self, value):
-        return float(value)
+        return float(value) if not self.meaningless(value) else None
 
 
 class IntegerValue(Value):
     field = forms.IntegerField
 
     def to_python(self, value):
-        return int(value)
+        return int(value) if not self.meaningless(value) else None
 
 
 class PercentValue(Value):
 
     class field(forms.DecimalField):
-
         def __init__(self, *args, **kwargs):
-            forms.DecimalField.__init__(self, 100, 0, 5, 2, *args, **kwargs)
+            forms.DecimalField.__init__(self, max_value=100, min_value=0, max_digits=5, decimal_places=2, *args, **kwargs)
 
         class widget(forms.TextInput):
             def render(self, *args, **kwargs):
                 # Place a percent sign after a smaller text field
                 attrs = kwargs.pop('attrs', {})
                 attrs['size'] = attrs['max_length'] = 6
-                return forms.TextInput.render(self, attrs=attrs, *args, **kwargs) + '%'
+                return mark_safe(forms.TextInput.render(self, attrs=attrs, *args, **kwargs) + ' %')
 
     def to_python(self, value):
-        return Decimal(value) / 100
+        return Decimal(value) / 100 if not self.meaningless(value) else None
 
 
 class PositiveIntegerValue(IntegerValue):
@@ -205,7 +207,7 @@ class TextValue(Value):
     field = forms.CharField
 
     def to_python(self, value):
-        return six.text_type(value)
+        return str(value)
 
 
 class EmailValue(Value):
@@ -213,7 +215,7 @@ class EmailValue(Value):
     field = forms.EmailField
 
     def to_python(self, value):
-        return six.text_type(value)
+        return str(value)
 
 
 class PasswordValue(Value):
@@ -221,6 +223,8 @@ class PasswordValue(Value):
         widget = forms.PasswordInput
 
         def __init__(self, **kwargs):
+            if kwargs['initial']:
+                kwargs['required'] = False
             if not kwargs.get('help_text'):
                 kwargs['help_text'] = _(
                     'Leave empty in order to retain old password. Provide new value to change.')
@@ -259,7 +263,7 @@ class MultiSeparatorValue(TextValue):
 
     def to_python(self, value):
         if value:
-            value = six.text_type(value)
+            value = str(value)
             value = value.split(self.separator)
             value = [x.strip() for x in value]
         else:
@@ -269,42 +273,38 @@ class MultiSeparatorValue(TextValue):
 
 class ImageValue(Value):
     def __init__(self, *args, **kwargs):
-        if 'upload_to' in kwargs:
-            self._upload_to = kwargs.pop('upload_to', '')
+        self._upload_to = kwargs.pop('upload_to', '')
+        self._delete_old = kwargs.pop('delete_old', True)
         super(ImageValue, self).__init__(*args, **kwargs)
 
     class field(forms.ImageField):
         class widget(forms.FileInput):
             "Widget with preview"
+            template_name = 'dbsettings/image_widget.html'
 
-            def render(self, name, value, attrs=None):
-                output = []
-
-                try:
-                    if not value:
-                        raise IOError('No value')
+            def render(self, name, value, attrs=None, renderer=None):
+                """Render the widget as an HTML string."""
+                context = self.get_context(name, value, attrs)
 
+                if value:
                     from PIL import Image
                     Image.open(value.file)
-                    file_name = pjoin(settings.MEDIA_URL, value.name).replace("\\", "/")
-                    params = {"file_name": file_name}
-                    output.append('<p><img src="%(file_name)s" width="100" /></p>' % params)
-                except IOError:
-                    pass
+                    context['image_url'] = pjoin(settings.MEDIA_URL, value.name).replace("\\", "/")
+                else:
+                    context['image_url'] = None
 
-                output.append(forms.FileInput.render(self, name, value, attrs))
-                return mark_safe(''.join(output))
+                return self._render(self.template_name, context, renderer)
 
     def to_python(self, value):
         "Returns a native Python object suitable for immediate use"
-        return six.text_type(value)
+        return str(value)
 
-    def get_db_prep_save(self, value):
+    def get_db_prep_save(self, value, oldvalue=None):
         "Returns a value suitable for storage into a CharField"
         if not value:
             return None
 
-        hashed_name = md5(six.text_type(time.time())).hexdigest() + value.name[-4:]
+        hashed_name = md5(str(time.time()).encode()).hexdigest() + value.name[-4:]
         image_path = pjoin(self._upload_to, hashed_name)
         dest_name = pjoin(settings.MEDIA_ROOT, image_path)
         directory = pjoin(settings.MEDIA_ROOT, self._upload_to)
@@ -315,7 +315,13 @@ class ImageValue(Value):
             for chunk in value.chunks():
                 dest_file.write(chunk)
 
-        return six.text_type(image_path)
+        # Delete old file
+        if oldvalue and self._delete_old:
+            old_dest_name = pjoin(settings.MEDIA_ROOT, oldvalue)
+            if os.path.exists(old_dest_name):
+                os.unlink(old_dest_name)
+
+        return str(image_path)
 
     def to_editor(self, value):
         "Returns a value suitable for display in a form widget"
@@ -350,8 +356,8 @@ class DateTimeValue(Value):
                 continue
         return None
 
-    def get_db_prep_save(self, value):
-        if isinstance(value, six.string_types):
+    def get_db_prep_save(self, value, **kwargs):
+        if isinstance(value, str):
             return value
         return value.strftime(self._formats[0])
 
